==> v10.5.1.zip: brewman/brewman/brewman/routers/client.py <==
import uuid

from typing import List

import brewman.schemas.client as schemas

from fastapi import APIRouter, HTTPException, Security, status
from sqlalchemy import delete, select
from sqlalchemy.exc import SQLAlchemyError

from ..core.security import get_current_active_user as get_user
from ..db.session import SessionFuture
from ..models.client import Client
from ..models.login_history import LoginHistory
from ..schemas.user import UserToken


router = APIRouter()


@router.put("/{id_}", response_model=None)
def update_route(
    id_: uuid.UUID,
    data: schemas.ClientIn,
    user: UserToken = Security(get_user, scopes=["clients"]),
) -> None:
    try:
        with SessionFuture() as db:
            item: Client = db.execute(select(Client).where(Client.id == id_)).scalar_one()
            item.enabled = data.enabled
            if item.enabled:
                item.otp = None
            item.name = data.name
            db.commit()
    except SQLAlchemyError as e:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )


@router.delete("/{id_}", response_model=None)
def delete_route(
    id_: uuid.UUID,
    user: UserToken = Security(get_user, scopes=["clients"]),
) -> None:
    try:
        with SessionFuture() as db:
            item: Client = db.execute(select(Client).where(Client.id == id_)).scalar_one()
            db.execute(delete(LoginHistory).where(LoginHistory.client_id == item.id))
            db.delete(item)
            db.commit()
    except SQLAlchemyError as e:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )


@router.get("/list", response_model=List[schemas.ClientList])
async def show_list(
    user: UserToken = Security(get_user, scopes=["clients"]),
) -> List[schemas.ClientList]:
    with SessionFuture() as db:
        list_ = db.execute(select(Client).order_by(Client.name)).scalars().all()
        clients = []
        for item in list_:
            clients.append(
                schemas.ClientList(
                    id=item.id,
                    code=item.code,
                    name=item.name,
                    enabled=item.enabled,
                    otp=item.otp,
                    creationDate=item.creation_date,
                    lastUser=item.login_history[0].user.name if len(item.login_history) else "None",
                    lastDate=item.login_history[0].date if len(item.login_history) else None,
                )
            )
        return clients


@router.get("/{id_}", response_model=schemas.Client)
def show_id(
    id_: uuid.UUID,
    user: UserToken = Security(get_user, scopes=["clients"]),
) -> schemas.Client:
    with SessionFuture() as db:
        item: Client = db.execute(select(Client).where(Client.id == id_)).scalar_one()
        return schemas.Client(
            id=item.id,
            code=item.code,
            name=item.name,
            enabled=item.enabled,
            otp=item.otp,
            creationDate=item.creation_date,
        )

==> v11.1.2.zip: brewman/brewman/brewman/routers/client.py <==
import uuid

import brewman.schemas.client as schemas

from fastapi import APIRouter, HTTPException, Security, status
from sqlalchemy import delete, select
from sqlalchemy.exc import SQLAlchemyError

from ..core.security import get_current_active_user as get_user
from ..db.session import SessionFuture
from ..models.client import Client
from ..models.login_history import LoginHistory
from ..schemas.user import UserToken


router = APIRouter()


@router.put("/{id_}", response_model=None)
def update_route(
    id_: uuid.UUID,
    data: schemas.ClientIn,
    user: UserToken = Security(get_user, scopes=["clients"]),
) -> None:
    try:
        with SessionFuture() as db:
            item: Client = db.execute(select(Client).where(Client.id == id_)).scalar_one()
            item.enabled = data.enabled
            if item.enabled:
                item.otp = None
            item.name = data.name
            db.commit()
    except SQLAlchemyError as e:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )


@router.delete("/{id_}", response_model=None)
def delete_route(
    id_: uuid.UUID,
    user: UserToken = Security(get_user, scopes=["clients"]),
) -> None:
    try:
        with SessionFuture() as db:
            item: Client = db.execute(select(Client).where(Client.id == id_)).scalar_one()
            db.execute(delete(LoginHistory).where(LoginHistory.client_id == item.id))
            db.delete(item)
            db.commit()
    except SQLAlchemyError as e:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )


@router.get("/list", response_model=list[schemas.ClientList])
async def show_list(
    user: UserToken = Security(get_user, scopes=["clients"]),
) -> list[schemas.ClientList]:
    with SessionFuture() as db:
        list_ = db.execute(select(Client).order_by(Client.name)).scalars().all()
        clients = []
        for item in list_:
            clients.append(
                schemas.ClientList(
                    id_=item.id,
                    code=item.code,
                    name=item.name,
                    enabled=item.enabled,
                    otp=item.otp,
                    creation_date=item.creation_date,
                    last_user=item.login_history[0].user.name if len(item.login_history) else "None",
                    last_date=item.login_history[0].date if len(item.login_history) else None,
                )
            )
        return clients


@router.get("/{id_}", response_model=schemas.Client)
def show_id(
    id_: uuid.UUID,
    user: UserToken = Security(get_user, scopes=["clients"]),
) -> schemas.Client:
    with SessionFuture() as db:
        item: Client = db.execute(select(Client).where(Client.id == id_)).scalar_one()
        return schemas.Client(
            id_=item.id,
            code=item.code,
            name=item.name,
            enabled=item.enabled,
            otp=item.otp,
            creation_date=item.creation_date,
        )
